@@ -5,16 +5,19 @@
 "diagram" <- function(fit,...) {
 # figure out which psych function was called and then call the appropriate diagram
 value <- NULL
-omega <- bassAck <- extend <- lavaan <- NULL    #weird requirement to avoid being global
-#if(length(class(fit)) == 1) {if (inherits(fit,"lavaan")) fn <- "lavaan" } 
+omega <- bassAck <- extend <- extension <-  lavaan <- fa.reg <-  NULL    #weird requirement to avoid being global
+ names <- cs(lm, lavaan,fa,principal,iclust,omega,bassAck, extension, extend, fa.reg, esem,mediate, lmCor, setCor, fa.multi,lm)
+if(length(class(fit)) == 1) {
+   value <- inherits(fit,names,which=TRUE) #if (inherits(fit,"lm")) value <- "lm" } 
+   if(any(value > 0) ) { value <- names[which(value > 0)]} else {value <- "other"}
+  }
 
-
- #if(length(class(fit)) > 1)  {
-    names <- cs(lavaan,fa,principal,iclust,omega,lavaan,bassAck, extend)
+if(length(class(fit)) > 1)  {
+    names <- cs(lm, lavaan,fa,principal,iclust,omega,bassAck, extension, extend, fa.reg, esem,mediate, lmCor, setCor, fa.multi,lm)
     value <- inherits(fit,names,which=TRUE)  # value <- class(x)[2]
-    if(any(value > 1) ) { value <- names[which(value > 0)]} else {value <- "other"}
+    if(any(value > 0) ) { value <- names[which(value > 0)]} else {value <- "other"}
     
-#     } else {value <- "other"}
+     } 
 
 switch(value,  #replaced a series of if statements to switch  12/23/18
 fa  = {fa.diagram(fit,...) },
@@ -23,7 +26,15 @@ iclust = {iclust.diagram(fit,...)},
 omega = {omega.diagram(fit,...)},
 lavaan =  {lavaan.diagram(fit,...)}, 
 bassAck = {bassAckward.diagram(fit,...)} ,
-extend = {extension.diagram(fit ,...)}
+extend = {extension.diagram(fit ,...)},
+extension ={extension.diagram(fit,...)},
+fa.reg ={extension.diagram(fit,...)},
+esem = {esem.diagram(fit,...)},
+setCor = {lmDiagram(fit,...)},
+lmCor = {lmDiagram(fit,...)},
+lm = {lmDiagram(fit, ...)},
+mediate ={mediate.diagram(fit,...)},
+fa.multi ={fa.multi.diagram(fit,...)}
 )
 }
 
@@ -36,8 +47,8 @@ extend = {extension.diagram(fit ,...)}
     yrange = (ylim[2] - ylim[1])
     xs <- .10 * xrange
     ys <- .10 * yrange
-     #len <- max(nchar(labels)*cex*.2/2,cex*.25)*xs
-     len <- pmax(strwidth(labels,units="user",cex=cex,...),strwidth("abc",units="user",cex=cex,...)) /1.8
+     #len <- max(nchar(labes)*cex*.2/2,cex*.25)*xs
+     len <- pmax(strwidth(labels,units="user",cex=cex,...),strwidth("abc",units="user",cex=cex,...)) /1.8   #was 1.8
      vert <- pmax(strheight(labels,units="user",cex=cex,...),strheight("ABC",units="user",cex=cex,...))/1.
     # vert <- min(cex*.3 * ys,.3)
     if(draw) rect(x-len,y-vert,x+len,y+vert)
@@ -91,6 +102,25 @@ extend = {extension.diagram(fit ,...)}
    return(xs)
 }
 
+"dia.ellipse.fill" <-    function (x,y,e.size=.05,xlim=c(0,1),ylim=c(0,1),draw=TRUE,density=NULL,angle=NULL,...) { 
+#code adapted from John Fox
+    segments=51
+    angles <- (0:segments) * 2 * pi/segments
+    angles <- c(angles,NA)   #this allows us to draw multiple ellipses
+    unit.circle <- cbind(cos(angles), sin(angles))
+    xrange = (xlim[2] - xlim[1])
+    yrange = (ylim[2] - ylim[1])
+    xs <- e.size * xrange
+    ys <- e.size * yrange
+    ellipse <- unit.circle 
+    ellipse[,1] <- ellipse[,1]*xs + x
+    ellipse[,2] <- ellipse[,2]*ys + y  #ys?
+   if(draw) lines(ellipse, ...)
+   polygon(ellipse,density=density,angle=angle,...)
+   # return(list(xs=xs,ys=ys))
+   return(xs)
+}
+
 #added to allow for fast drawing of multiple ellipses in fa.diagram and bassAckward diagram
 dia.multi.ellipse <-  function(x, y = NULL, labels = NULL, cex = 1,e.size=.4,xlim=c(0,1),ylim=c(0,1),draw=TRUE, ...) {
  segments=51
@@ -100,13 +130,20 @@ dia.multi.ellipse <-  function(x, y = NULL, labels = NULL, cex = 1,e.size=.4,xli
     unit.circle <- cbind(cos(angles), sin(angles))
     xrange = (xlim[2] - xlim[1])
     yrange = (ylim[2] - ylim[1])
-   # xs <- e.size * xrange
+  
   
     ys <- max(strheight(labels,units="user"))  #*.55
+  
+    # xs <- e.size * xrange
+   #ys <- e.size * yrange
     ellipse <- unit.circle 
 
   
    xs  <- strwidth(labels,units="user")*.6 #this makes the ellipse closer to left and right
+
+   #xs <- xs * e.size
+   xs <- xs * cex
+   ys <- ys * cex
      ellipsex <- rep(x,each=(segments + 2)) + unit.circle[,1] * rep(xs,each=segments +2)
      ellipsey <- rep(y,each=(segments + 2))  + unit.circle[,2] *ys
      
@@ -137,70 +174,8 @@ dia.multi.ellipse <-  function(x, y = NULL, labels = NULL, cex = 1,e.size=.4,xli
     dia.rect <- list(left=left,right=right,top=top,bottom=bottom,center=c(x,y),radius=radius)
      }
      
-     
-"dia.arrow" <- 
-function(from,to,labels=NULL,scale=1,cex=1,adj=2, both=FALSE,pos=NULL,l.cex,gap.size=NULL,draw=TRUE,col="black",lty="solid",...) {
-    if(missing(gap.size)) gap.size <- .2
-    if(missing(l.cex)) l.cex <- cex
-    text.values <- NULL
-    radius1 <- radius2 <- 0
-   if(!is.list(to)) {tocenter <- to} else {tocenter <- to$center}
- 	if(is.list(from)) {if(!is.null(from$radius)) {radius1 <- from$radius
- 	        radius2 <- 0}
- 	      #  if((tocenter[2] < from$center[2]) & ((tocenter[1] == from$center[1]))  ) {from <- from$bottom} else {
- 	        if(!is.null(from$center)) from <- from$center}
-    		
-    		
-       if(is.list(to)) {if(!is.null(to$radius)) {radius2 <- to$radius 
-       			to <- to$center}}
-       theta <- atan((from[2] - to[2])/(from[1] - to[1]))
-       costheta <- cos(theta)
-       sintheta <- sin(theta) 
-       dist <- sqrt((to[1] - from[1])^2 + (to[2] - from[2])^2)
-        if((adj  > 3 ) || (adj < 1)) {
-                   x <- (to[1] + from[1])/2
-                   y <- (to[2] + from[2])/2
-                   } else {
-        
-       x <- from[1] - sign(from[1] -to[1]+.001) *(4-adj) * costheta * dist/4
-      
-       y <- from[2] -  sign(from[1]-to[1] +.001)* (4-adj)  *  sintheta * dist/4}  #this is the problem when line is vertical
-      #x <- from[1] - sign(from[1]-to[1]) *adj *  cos(theta) * dist/6
-      #y <- from[2] -  sign(from[1]-to[1])* adj *  sin (theta)* dist/6}
-       
-     #   if(is.null(labels)) {h.size <- 0 } else{ h.size <- nchar(labels)*cex*.15}
-      #  if(is.null(labels)) {v.size <- 0 } else{ v.size <- cex * .1}
-        
-       if(is.null(labels)) {h.size <- 0 } else{ h.size <- nchar(labels)*cex*gap.size}
-       if(is.null(labels)) {v.size <- 0 } else{ v.size <- cex * .7 * gap.size}
-     
-        if(from[1] <  to[1] ) {h.size <- -h.size
-                               radius1 <- -radius1
-                               radius2 <- -radius2}
-        x0 <- from[1] -costheta * radius1
-        xr <- x + h.size * costheta* v.size
 
-        y0 <- from[2] - sintheta * radius1  ##
-       
-        xl <- x - h.size * costheta* v.size
-        yr <- y + v.size * sintheta * h.size 
-        yl <- y - v.size * sintheta  *h.size
-        xe <- to[1] +costheta * radius2
-        ye <- to[2] + sintheta * radius2
-      if(!is.null(labels))  {  if(draw){ text(x,y,labels,cex=l.cex,pos=pos,...)
-                           text.values <- NULL } else {
-                           if(is.null(pos)) pos <- 0
-                           text.values <- c(x,y,labels,pos=pos,cex=l.cex) } }
-      
-       if(draw) {arrows(x0,y0,xr,yr, length = (both+0) * .1*scale, angle = 30, code = 1, ...)
-            arrows(xl,yl,xe,ye, length = 0.1*scale, angle = 30, code = 2,...)
-            arrow.values <- NULL
-            } else {
-        
-        arrow.values <- list(x0,y0,xr,yr,  length = (both+0) * .1*scale, angle = 30, code = 1, xl,yl,xe,ye, length2 = 0.1*scale, angle2 = 30, code2 = 2,col=col,lty=lty)
-        }
-        invisible(list(text.values=text.values,arrow.values=arrow.values))
-       }       
+     
 
        
        
@@ -373,7 +348,7 @@ if(side == 4)    {x <- c(location$right[1],location$right[1]+scale,location$righ
      if (shape==3) {dia.triangle(x, y = NULL, labels =NULL,  cex = 1, xlim=c(0,1),ylim=c(0,1),...)}
      }
      
- "multi.curved.arrow" <- function(curved.list,...) {
+ "multi.curved.arrow" <- function(curved.list,l.cex=NULL,...) {
 
 leng  <- length(curved.list)
 if(leng > 0) {
@@ -387,7 +362,8 @@ textvalues <- matrix(unlist(curved.list[seq+5]),ncol=3,byrow=TRUE)
 
 lines(firstX,firstY)
 lines(secondX,secondY)
-text(x= textvalues[,1],y = textvalues[,2],labels=textvalues[,3])
+if(is.null(l.cex) ) l.cex=1
+text(x= textvalues[,1],y = textvalues[,2],labels=textvalues[,3],cex=l.cex,...)
  }}
  
  "multi.self" <- function(self.list,...) {
@@ -425,12 +401,23 @@ text(x= textvalues[,1],y = textvalues[,2],labels=textvalues[,3])
 
 }
    
-   
-"multi.rect" <- function(rect.list,...) {
-  nvar <- length(rect.list)/7
- tv <- matrix(unlist(rect.list),nrow=nvar,byrow=TRUE)
-      all.rects.x <- as.numeric(tv[,6]) #the center of the figure
-      all.rects.y <- as.numeric(tv[,3])
-      all.rects.names <-  tv[,1]
-      dia.rect(all.rects.x, all.rects.y,all.rects.names) 
-}  
+ 
+ 
+ multi.rect <-  function (rect.list, ...) 
+{
+    nvar <- length(rect.list)/7
+    tv <- matrix(unlist(rect.list), nrow = nvar, byrow = TRUE)
+    all.rects.x <- as.numeric(tv[, 6])
+    all.rects.y <- as.numeric(tv[, 3])
+    all.rects.names <- tv[, 1]
+    dia.rect(all.rects.x, all.rects.y, all.rects.names)
+}
+  
+# "multi.rect" <- function(rect.list,...) {
+#   nvar <- length(rect.list)
+#  tv <- matrix(unlist(rect.list),nrow=nvar,byrow=TRUE)
+#       all.rects.x <- as.numeric(tv[,5]) #the center of the figure
+#       all.rects.y <- as.numeric(tv[,2])
+#       all.rects.names <-  tv[,1]    # i dont have names
+#       dia.rect(all.rects.x, all.rects.y,all.rects.names) 
+# }  
