@@ -32,11 +32,25 @@ function(R,digits=2,minlength=5) {
 	invisible(R[lower.tri(R,diag=FALSE)])}
 	
 "lowerCor" <- 
-function(x,digits=2,use="pairwise",method="pearson",minlength=5) {
+function(x,digits=2,use="pairwise",method="pearson",minlength=5,cor="cor",show=TRUE) {
 nvar <- NCOL(x)
 x <- char2numeric(x)    #added 1/2/21
-   R <- cor(x,use=use,method=method)
-   lowerMat(R,digits,minlength=minlength)
+if(cor %in% c("tetrachoric","tetra","binary")) cor<- "tetrachoric"
+if(cor %in% c("polychoric","poly")) cor<- "polychoric"
+switch(cor, cor = {
+            R <- cor(x, use = use, method = method)
+        }, tetrachoric = {
+            R <- tetrachoric(x)$rho
+        }, polychoric = {
+            R <- polychoric(x)$rho
+        }, cov = {
+            R <- cov(x, use = use)
+        }, mixed = {
+            R <- mixedCor(x)$rho
+        })
+
+  # R <- cor(x,use=use,method=method)
+   if(show) lowerMat(R,digits,minlength=minlength)
    invisible(R)
    }
 
@@ -136,8 +150,8 @@ r <- cor(x[1:(n.obs-lag)],x[(lag+1):n.obs],use=use)
 x <- as.matrix(x)
 n <- colSums(!is.na(x))
 mssd <- colSums(diff(x,lag=lag,na.rm=na.rm)^2,na.rm=na.rm)/(n-lag)
-v1 <- apply(x[1:(n.obs-lag),],2,sd, na.rm=na.rm)^2
-v2 <- apply(x[(lag+1):n.obs,],2, sd,na.rm=na.rm)^2
+v1 <- apply(x[1:(n.obs-lag),,drop=FALSE],2,sd, na.rm=na.rm)^2
+v2 <- apply(x[(lag+1):n.obs,,drop=FALSE],2, sd,na.rm=na.rm)^2
 # r <- -(mssd - v1 - v2)/(2*sqrt(v1*v2))
 r <- diag(cor(x[1:(n.obs-lag),],x[(lag+1):n.obs,],use=use))
  result <- list(autoR = r,rssd=sqrt(mssd))
@@ -169,6 +183,8 @@ return(result)}
 
 
 
+
+
 #####
 sim.mssd <- function(n,r,g=.1) {
 rw <- rnorm(n,sqrt(1/(1-r^2)))
@@ -279,7 +295,7 @@ return(y)}
    rownames(f) <- ord
    if(!is.null(dictionary))  { 
    contents <- lookup(rownames(f),dictionary)} else {message("fa.lookup requires a dictionary, otherwise just use fa.sort")}
-   if(!is.null(h2)) {results <- data.frame(round(unclass(f),digits=digits),com=round(com,digits=digits),h2=round(h2,digits=digits))} else {
+   if(!is.null(h2)) {results <- data.frame(round(unclass(f),digits=digits),h2=round(h2,digits=digits),com=round(com,digits=digits))} else {
    results <- data.frame(round(unclass(f),digits=digits))}
    
    results <- merge(results,contents,by="row.names",all.x=TRUE,sort=FALSE)
@@ -404,7 +420,19 @@ if(is.null(names(fa.results)) )  {temp <- fa.results   #the matrix form
      return(contents)
     }
   
-  "item.lookup" <- 
+  #created 1/15/24
+"itemSort" <- function(m, dictionary, ascending=TRUE, digits = 2) {
+ contents <- lookup(rownames(m), y=dictionary)
+ results <- data.frame(means=round(m,digits=digits),contents)
+ results <- psychTools::dfOrder(results,ascending=ascending)
+ return(results)
+}
+ 
+
+
+   
+ #patched   1/15/24 to give means  
+  "item.lookup" <-             
 function (f,m, dictionary,cut=.3, digits = 2) {
    # f <- fa.sort(f)
     none<- NULL   #A strange requirement of R 4.0
@@ -445,7 +473,9 @@ function (f,m, dictionary,cut=.3, digits = 2) {
         ord <- rownames(f)
         nfact <- ncol(f)
     }
-    means <- m[ord]   #incorrectly added a comma to allow it sort dataframes 6/22/21
+   if(!is.null(dim(m))) {
+    means <- m[ord,,drop=FALSE]   #fixed 1/15/24  
+    } else {means <- m}
     f <- data.frame(unclass(f),means=means)
 
     contents <- lookup(rownames(f), y=dictionary)
@@ -471,7 +501,7 @@ function (f,m, dictionary,cut=.3, digits = 2) {
     return(res)
 }
 
-setCorLookup<- function(x,dictionary=NULL,cut=0,digits=2,p=.05) {
+lmCorLookup <- setCorLookup <- function(x,dictionary=NULL,cut=0,digits=2,p=.05) {
 coef <- x$coefficients
 probs <- x$Probability
 labels <- dictionary[rownames(coef),,drop=FALSE]
@@ -583,18 +613,38 @@ text(x,y-2,"Unreliable but Valid")
 
 #finally added the char2numeric so it will not choke on character variables   1/3/21
 
-"cor2" <- function(x,y=NULL,digits=2,use="pairwise",method="pearson") {
+#added the cor option 11/18/23
+"cor2" <- function(x,y=NULL,digits=2,use="pairwise",method="pearson",cor="cor",show=TRUE) {
 multi <- FALSE
 if(is.list(x) && is.null(y)) {multi <- TRUE
  n <- length(x)
 xi <- x[[1]]
  for (i in 2:n) {xi <- cbind(xi,x[[i]])}
-  
-R <- cor(xi,use=use,method=method) }else {
+
+switch(cor,
+cor= {R <- cor(xi,use=use,method=method) },
+tetrachoric = {R <- tetrachoric(xi)$rho},
+polychoric= {R <- polychoric(xi)$rho},
+cov = {R <- cov(xi, use=use)},
+mixed = {R <- mixedCor(xi)$rho}
+)
+}else {
 x <- char2numeric(x)
 y <- char2numeric(y)
-R <- cor(x,y,use=use,method=method)}
+
+switch(cor,
+
+cor= {R <- cor(x,y,use=use,method=method)},
+tetrachoric = {R <- tetrachoric(x,y)$rho},
+polychoric= {R <- polychoric(x,y)$rho},
+cov = {R <- cov(x,y, use=use)},
+mixed = {R <- mixedCor(cbind(x,y))$rho}
+
+)
+}
+if(show) {
 if(multi) {lowerMat(R,digits) } else {print(round(R,digits))}
+}
 invisible(R)}
 
 levels2numeric <- function(x) {
@@ -621,7 +671,19 @@ pretty}
 #this has a downsize that it converts numbers stored as characters to factors (see nchar2numeric)
 #added the flag option and the change the colname option 1/2/21
 
+"isAllNumeric" <- function(x){
+      nvar <- NCOL(x)
+      bad <- rep(0,nvar)
+ for(i in 1:nvar) { 
+     if(!is.numeric(x[[i]]) ){if(is.factor(unlist(x[[i]]))| is.character(unlist(x[[i]]))    
+     ) bad [i] <- 1
+     }}
+     if(prod(bad)==0) {result <- TRUE} else {result<- bad}
+     invisible(result)
+     }
+
 "char2numeric" <- function(x,flag=TRUE) {
+
  nvar <- NCOL(x)
  for(i in 1:nvar) {   
         if(!is.numeric(x[[i]] ))  {
@@ -691,18 +753,19 @@ flip <- function(R,key) {#reverse correlations with keys  < 0
 	R[,key<0] <- -R[,key < 0]
   return(R)}
     
-
- matMult <- function(A,B) {  #matrix multiplication without matrices!  
-  nvar1 <- ncol(A)
-  nvar2 <- ncol(B)
-  M <- matrix(NA,ncol=nvar1,nrow=nrow(B))
-    if(nrow(A) != nvar2) {stop("incompatible dimensions")}
-   for(i in 1:nvar1) {
-    for (j in 1:nrow(B))  {
-    M[j,i] <-  sum(A[,i] * B[j,], na.rm=TRUE)
+#do matrix multiplication with missing values
+ matMult <- function(x,y) {  #matrix multiplication without matrices!  
+  nvar1 <- NCOL(x)
+  nvar2 <- NROW(y)
+  M <- matrix(NA,ncol =NCOL(y),nrow=NROW(x))
+    if(NCOL(x) != NROW(y)) {stop("incompatible dimensions")}
+   for(i in 1:NROW(x)) {
+    for (j in 1:NCOL(y))  {
+    M[i,j] <-  sum(x[i,] * y[,j], na.rm=TRUE)
     } 
     }
-    return(M)}
+    return(M)
+    }
      
    
    
@@ -716,6 +779,7 @@ flip <- function(R,key) {#reverse correlations with keys  < 0
            
     scores[,factors] <-t(colMeans( weights[,factors] * titem,  na.rm=TRUE))
     }
+    scores<- scale(scores)  #return standardized values
     return(scores)
   }
 
@@ -729,4 +793,17 @@ for (i in 1:n) {
  return(count)
  }
 
+#Create SAPA data by sampling
+SAPAfy <- function(x,y)
+{
+  n_vars <- ncol(x)
+  n_questions_unanswered <- round(n_vars - y)  #  in case someone inputs a double.
+  
+  for (xxx in 1:nrow(x)) 
+  {
+    x[xxx, sample.int(n_vars, n_questions_unanswered)  ] <- NA 
+  }
+  return(x)
+}
+
 
