@@ -1,4 +1,4 @@
-"print.psych.omega" <- 
+"print_psych.omega" <- 
 function(x,digits=2,all=FALSE,cut=NULL,sort=FALSE,...) { 
 xx <- x
 if(!is.null(x$ci)) {
@@ -46,7 +46,7 @@ if(is.null(cut)) cut <- .2
 	   
        numfactors <- dim(x$schmid$sl)[2] -3
        eigenvalues <- diag(t(x$schmid$sl[,1:numfactors]) %*% x$schmid$sl[,1:numfactors])
-       cat("\nWith eigenvalues of:\n")
+       cat("\nWith Sums of squares  of:\n")
        ev.rnd <- round(eigenvalues,digits)
        print(ev.rnd,digits=digits)
       
@@ -112,14 +112,14 @@ if(is.null(cut)) cut <- .2
    
    
    
-   "print.psych.omegaSem" <- 
+   "print_psych.omegaSem" <- 
    function(x,digits=2,all=FALSE,cut=NULL,sort=FALSE,...) { 
      if(is.null(cut)) cut <- .2
 	 cat( x$title,"\n") 
 	 if(!is.null(x$Call)) {# we have run this from omegaSem so we should print omega results first
 	 cat("Call: ")
      print(x$Call)
-   print.psych.omega(x$omegaSem,digits=digits,all=all,cut=cut,sort=sort,...)
+   print_psych.omega(x$omegaSem,digits=digits,all=all,cut=cut,sort=sort,...)
    x <- x$omega.efa
    } 
    loads <- x$cfa.loads
@@ -157,7 +157,7 @@ if(is.null(cut)) cut <- .2
 	    
 	   loads <- as.matrix(load.2) 
 	   eigenvalues <- diag(t(loads) %*% loads)
-       cat("\nWith eigenvalues of:\n")
+       cat("\nWith sum of squared loadings of:\n")
        ev.rnd <- round(eigenvalues,digits)
        print(ev.rnd,digits=digits)
        
