@@ -37,10 +37,6 @@ if(is.list(keys) | is.character(keys)){ keys <- make.keys(items,keys)
 return(x)
 }
  
- "recode" <- function(x,where,isvalue,newvalue) {
-  if (missing(where))  where <-1:NCOL(x)
-  x[,where==isvalue] <- newvalue
-   }
  
 "scrub" <- 
  function (x, where, min, max, isvalue,newvalue,cuts=NULL) 
@@ -87,3 +83,4 @@ return(x)
 
 
 
+
